@@ -1,4 +1,20 @@
 # Class 'MVNresponse' (multivariate normal response model)
+
+cov2par <- function(x) {
+	if(NROW(x) != NCOL(x)) stop("cov2par requires a square matrix") 
+	x[lower.tri(x,diag=TRUE)]
+}
+
+par2cov <- function(x) {
+	npar <- length(x)
+	dim <- (sqrt(8*npar + 1) - 1)/2
+	if(abs(dim - round(dim)) >= .Machine$double.eps^0.5) stop("number of parameters not suitable for par2cov")
+	cov <- matrix(0.0,ncol=dim,nrow=dim)
+	cov[lower.tri(cov,diag=TRUE)] <- x
+	cov[upper.tri(cov)] <- t(cov)[upper.tri(cov)]
+	cov
+}
+
 setClass("MVNresponse",
   representation(formula="formula"),
   contains="response"
@@ -10,7 +26,7 @@ setMethod("fit","MVNresponse",
 		pars <- object@parameters
 		if(!is.null(w)) fit <- lm.wfit(x=object@x,y=object@y,w=w) else fit <- lm.fit(x=object@x,y=object@y)
 		object@parameters$coefficients <- fit$coefficients
-		if(!is.null(w)) object@parameters$Sigma <- vech(cov.wt(x=fit$residuals,wt=w)$cov) else object@parameters$Sigma <- vech(cov(fit$residuals))
+		if(!is.null(w)) object@parameters$Sigma <- cov2par(cov.wt(x=fit$residuals,wt=w)$cov) else object@parameters$Sigma <- cov2par(cov(fit$residuals))
 		object
 	}
 )
@@ -23,7 +39,10 @@ dm_dmvnorm <- function(x,mean,sigma,log=FALSE,logdet,invSigma) {
         x <- matrix(x, ncol = length(x))
     }
     if (missing(mean)) {
-        mean <- rep(0, length = ncol(x))
+        mean <- matrix(0, ncol = ncol(x))
+    }
+    if(is.vector(mean)) {
+    	mean <- matrix(mean, ncol = ncol(x))
     }
     if(missing(invSigma)) {
     	if (missing(sigma)) {
@@ -32,7 +51,6 @@ dm_dmvnorm <- function(x,mean,sigma,log=FALSE,logdet,invSigma) {
     	invSigma <- solve(sigma)
     }
 	# check consistency
-	
 	if (NCOL(x) != NCOL(invSigma)) {
 	    stop("x and sigma have non-conforming size")
 	}
@@ -42,11 +60,15 @@ dm_dmvnorm <- function(x,mean,sigma,log=FALSE,logdet,invSigma) {
 	if (NCOL(invSigma) != NCOL(mean)) {
 		stop("mean and sigma have non-conforming size")
 	}
+	if(missing(logdet)) {
+		ev <- eigen(sigma, symmetric = TRUE, only.values = TRUE)$values
+		if(!all(ev >= 0)) return(rep(NaN,nrow(x))) else logdet <- sum(log(ev))
+	}
 	if(NROW(mean) == NROW(x)) {
 		# varying means
 		
 		# from "mahalanobis":    
-		x <- as.matrix(x) - mean
+		x <- x - mean
     	distval <- rowSums((x %*% invSigma) * x)
     	#names(retval) <- rownames(x)
    	 	#retval
@@ -57,7 +79,6 @@ dm_dmvnorm <- function(x,mean,sigma,log=FALSE,logdet,invSigma) {
 		}
 		distval <- mahalanobis(x, center = mean, cov = invSigma, inverted=TRUE)
 	}	
-    if(missing(logdet)) logdet <- sum(log(eigen(sigma, symmetric = TRUE, only.values = TRUE)$values))
     logretval <- -(ncol(x) * log(2 * pi) + logdet + distval)/2
     if (log) {
         return(logretval)
@@ -66,16 +87,15 @@ dm_dmvnorm <- function(x,mean,sigma,log=FALSE,logdet,invSigma) {
     }
 }
 
-
 setMethod("logDens","MVNresponse",
 	function(object,...) {
-		dm_dmvnorm(x=object@y,mean=predict(object),sigma=xpnd(object@parameters$Sigma),log=TRUE,...)
+		dm_dmvnorm(x=object@y,mean=predict(object),sigma=par2cov(object@parameters$Sigma),log=TRUE,...)
 	}
 )
 
 setMethod("dens","MVNresponse",
 	function(object,log=FALSE,...) {
-		dm_dmvnorm(x=object@y,mean=predict(object),sigma=xpnd(object@parameters$Sigma),log=log,...)
+		dm_dmvnorm(x=object@y,mean=predict(object),sigma=par2cov(object@parameters$Sigma),log=log,...)
 	}
 )
 
@@ -96,11 +116,11 @@ setMethod("simulate",signature(object="MVNresponse"),
 			mu <- predict(object)[times,]
 		}
 		nt <- nrow(mu)
-		if(nrow(object@parameters$coefficients==1)) response <- mvrnorm(nt*nsim,mu=mu[1,],Sigma=xpnd(object@parameters$Sigma))
+		if(nrow(object@parameters$coefficients==1)) response <- mvrnorm(nt*nsim,mu=mu[1,],Sigma=par2cov(object@parameters$Sigma))
 		else {
 			response <- matrix(0,nrow(mu),ncol(mu))
 			for(i in 1:nrow(mu)) {
-				response[i,] <- response <- mvrnorm(1,mu=mu[i,],Sigma=xpnd(object@parameters$Sigma))
+				response[i,] <- response <- mvrnorm(1,mu=mu[i,],Sigma=par2cov(object@parameters$Sigma))
 			}
 		}
 		return(response)
@@ -122,15 +142,25 @@ setMethod("MVNresponse",
 		if(!is.matrix(y)) y <- matrix(y,ncol=1)
 		parameters <- list()
 		parameters$coefficients <- matrix(0.0,ncol=ncol(y),nrow=ncol(x))
-		parameters$Sigma <- vech(diag(ncol(y)))
-		npar <- length(unlist(parameters))		
+		parameters$Sigma <- cov2par(diag(ncol(y)))
+		npar <- length(unlist(parameters))
+		parlow.coeff=rep(-Inf,length(unlist(parameters$coefficients)))
+		parup.coeff=rep(Inf,length(unlist(parameters$coefficients)))
+		parup.cov <- rep(Inf,length(unlist(parameters$Sigma)))
+		mcov <- matrix(-Inf,ncol(y),ncol(y))
+		diag(mcov) <- .Machine$double.eps
+		parlow.cov <- cov2par(mcov)
+		constr <- list(	
+			parup = c(parup.coeff,parup.cov),
+			parlow = c(parlow.coeff,parlow.cov)
+		)
 		if(is.null(fixed)) fixed <- as.logical(rep(0,npar))
 		if(!is.null(pstart)) {
 			if(length(pstart)!=npar) stop("length of 'pstart' must be",npar)
 			parameters$coefficients <- matrix(pstart[1:(ncol(x)*ncol(y))],ncol(x),byrow=T)
 			parameters$Sigma <- as.numeric(pstart[(length(parameters$coefficients)+1):length(pstart)])			
 		}
-		mod <- new("MVNresponse",formula=formula,parameters=parameters,fixed=fixed,x=x,y=y,npar=npar)
+		mod <- new("MVNresponse",formula=formula,parameters=parameters,fixed=fixed,x=x,y=y,npar=npar,constr=constr)
 		mod
 	}
 )
@@ -142,7 +172,7 @@ setMethod("show","MVNresponse",
 		cat("Coefficients: \n")
 		print(object@parameters$coefficients)
 		cat("Sigma: \n")
-		print(xpnd(object@parameters$Sigma))
+		print(par2cov(object@parameters$Sigma))
 	}
 )
 
