@@ -53,7 +53,7 @@ setMethod("simulate",signature(object="NORMresponse"),
       mu <- predict(object)[times]
     }  
     nt <- length(mu)
-    sd <- getpars(object)["sd"]
+    sd <- object@parameters$sd
     response <- rnorm(nt*nsim,mean=mu,sd=sd)
     #if(nsim > 1) response <- matrix(response,ncol=nsim)
     response <- as.matrix(response)
